﻿//INSTANT C# NOTE: Formerly VB project-level imports:
using System;
using System.Collections;
using System.Collections.Generic;
using System.Data;
using System.Drawing;
using System.Diagnostics;
using System.Windows.Forms;
using System.Linq;
using System.Xml.Linq;

using System.Data.OleDb;

namespace WindowsApplication1
{
	public partial class Form1
	{

		internal Form1()
		{
			InitializeComponent();
		}

	#region  Fields 

		/// <summary>
		/// Contains the tables and the relation between them.
		/// </summary>
		private DataSet data = new DataSet();

		/// <summary>
		/// Connects to the database.
		/// </summary>
		private OleDbConnection connection = new OleDbConnection(@"Provider=Microsoft.Jet.OLEDB.4.0;Data Source=|DataDirectory|\ParentChild.mdb;User Id=admin;Password=;");

	    /// <summary>
	    /// Retrieve data from the database and save changes back to the database.
	    /// </summary>
	    /// <remarks>
	    /// The WithEvents keyword is required so that the adapters can be included in the Handles clause of their RowUpdated event handlers.
	    /// </remarks>
	    private OleDbDataAdapter parentDataAdapter;
		private OleDbDataAdapter childDataAdapter;

		/// <summary>
		/// Generate the InsertCommand, UpdateCommand and DeleteCommand for the adapters when their Update method is called.
		/// </summary>
		private OleDbCommandBuilder parentCommandBuilder;
		private OleDbCommandBuilder childCommandBuilder;

	#endregion //Fields

	#region  Event Handlers 

		private void Form1_Load(object sender, System.EventArgs e)
		{
			try
			{
                BuildDataAccessObjects();
				LoadSchema();
				ConfigureAutoIncrements();
				ConfigureRelation();
				LoadData();
				BindData();
			}
			catch (Exception ex)
			{
				MessageBox.Show(ex.ToString(), "Error");
			}
		}

		/// <summary>
		/// Raised after a parent row is saved to the database.
		/// </summary>
		/// <param name="sender">
		/// The adapter that saved the row.
		/// </param>
		/// <param name="e">
		/// The data for the event.
		/// </param>
		/// <remarks>
		/// This event handler is used to retrieve an auto-generated ID from the database after a row is inserted and update the corresponding row in the local data set.
		/// </remarks>
		private void parentDataAdapter_RowUpdated(object sender, OleDbRowUpdatedEventArgs e)
		{
			//We are only interested in new records.
			if (e.StatementType == StatementType.Insert)
			{
				//Get the last ID auto-generated by the database.
				using (OleDbCommand command = new OleDbCommand("SELECT @@IDENTITY", this.connection))
				{
					//Update the ID of the local row.
					e.Row["ParentID"] = Convert.ToInt32(command.ExecuteScalar());
				}
			}
		}

		/// <summary>
		/// Raised after a child row is saved to the database.
		/// </summary>
		/// <param name="sender">
		/// The adapter that saved the row.
		/// </param>
		/// <param name="e">
		/// The data for the event.
		/// </param>
		/// <remarks>
		/// This event handler is used to retrieve an auto-generated ID from the database after a row is inserted and update the corresponding row in the local data set.
		/// </remarks>
		private void childDataAdapter_RowUpdated(object sender, OleDbRowUpdatedEventArgs e)
		{
			//We are only interested in new records.
			if (e.StatementType == StatementType.Insert)
			{
				//Get the last ID auto-generated by the database.
				using (OleDbCommand command = new OleDbCommand("SELECT @@IDENTITY", this.connection))
				{
					//Update the ID of the local row.
					e.Row["ChildID"] = Convert.ToInt32(command.ExecuteScalar());
				}
			}
		}

		/// <summary>
		/// Raised when the user clicks the Save button on the parent tool bar.
		/// </summary>
		/// <param name="sender">
		/// The Save button.
		/// </param>
		/// <param name="e">
		/// The data for the event.
		/// </param>
		/// <remarks>
		/// Saves all parent and child data.
		/// </remarks>
		private void SaveToolStripButton_Click(object sender, System.EventArgs e)
		{
			if (this.Validate())
			{
				this.parentBindingSource.EndEdit();
				this.childBindingSource.EndEdit();

				this.parentDataAdapter.Update(this.data, "Parent");
				this.childDataAdapter.Update(this.data, "Child");
			}
		}

	#endregion //Event Handlers

	#region  Methods

        private void BuildDataAccessObjects()
        {
            this.parentDataAdapter = new OleDbDataAdapter("SELECT * FROM Parent", this.connection)
                                         {MissingSchemaAction = MissingSchemaAction.AddWithKey};
            this.childDataAdapter = new OleDbDataAdapter("SELECT * FROM Child", this.connection)
                                        {MissingSchemaAction = MissingSchemaAction.AddWithKey};

            this.parentCommandBuilder = new OleDbCommandBuilder(this.parentDataAdapter)
                                            {QuotePrefix = "[", QuoteSuffix = "]"};
            this.childCommandBuilder = new OleDbCommandBuilder(this.childDataAdapter)
                                           {QuotePrefix = "[", QuoteSuffix = "]"};

            this.parentDataAdapter.RowUpdated += parentDataAdapter_RowUpdated;
            this.childDataAdapter.RowUpdated += childDataAdapter_RowUpdated;
        }

		/// <summary>
		/// Build the data set schema without actually retrieving any data.
		/// </summary>
		private void LoadSchema()
		{
			this.parentDataAdapter.FillSchema(this.data, SchemaType.Source, "Parent");
			this.childDataAdapter.FillSchema(this.data, SchemaType.Source, "Child");
		}

		/// <summary>
		/// Configure the auto-increment properties of the ID columns of both tables.
		/// </summary>
		private void ConfigureAutoIncrements()
		{
			ConfigureAutoIncrement(this.data.Tables["Parent"].Columns["ParentID"]);
			ConfigureAutoIncrement(this.data.Tables["Child"].Columns["ChildID"]);
		}

		/// <summary>
		/// Configure the auto-increment properties of a column.
		/// </summary>
		/// <remarks>
		/// The column will be configured to generate local IDs that will never class with those generated by the database.
		/// Database IDs will start at 1 and increase sequentially while local IDs will start at 0 and decrease sequentially.
		/// </remarks>
		private void ConfigureAutoIncrement(DataColumn column)
		{
				//The column should automatically generate local IDs.
			column.AutoIncrement = true;

				//The first local ID value generated should be 0.
				//The database IDs will start at 1, so local IDs will not clash.
			column.AutoIncrementSeed = 0;

				//The local IDs should each be 1 less than the previous.
				//The database IDs will each be 1 more than the previous, so local IDs will not clash.
			column.AutoIncrementStep = -1;
		}

		/// <summary>
		/// Configure the foreign key relation between the ParentID columns in the Parent and Child tables.
		/// </summary>
		/// <remarks>
		/// The relation is set to cascade updates, which means that when the local ID in a parent
		/// row is replaced with the database ID, all related child rows will be updated as well.
		/// </remarks>
		private void ConfigureRelation()
		{
			this.data.Relations.Add("ParentChild", this.data.Tables["Parent"].Columns["ParentID"], this.data.Tables["Child"].Columns["ParentID"], true).ChildKeyConstraint.UpdateRule = Rule.Cascade;
		}

		/// <summary>
		/// Retrieve the data from the database.
		/// </summary>
		/// <remarks></remarks>
		private void LoadData()
		{
			this.parentDataAdapter.Fill(this.data, "Parent");
			this.childDataAdapter.Fill(this.data, "Child");
		}

		/// <summary>
		/// Displays the data in the grids.
		/// </summary>
		/// <remarks>
		/// The child grid is bound such that it will only display the records related to the selected parent record.
		/// </remarks>
		private void BindData()
		{
			this.parentBindingSource.DataSource = this.data.Tables["Parent"];
			this.childBindingSource.DataMember = "ParentChild";
			this.childBindingSource.DataSource = this.parentBindingSource;

			this.parentDataGridView.DataSource = this.parentBindingSource;
			this.childDataGridView.DataSource = this.childBindingSource;
		}

	#endregion //Methods


	private static Form1 _DefaultInstance;
	public static Form1 DefaultInstance
	{
		get
		{
			if (_DefaultInstance == null)
				_DefaultInstance = new Form1();

			return _DefaultInstance;
		}
	}
	}

}